/*
 *
 * Copyright 2018, Theodore Chabardes
 * <theodore.chabardes@mines-paristech.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __BENCH_HPP__
#define __BENCH_HPP__

#include "offset.hpp"
#include <limits>
#include <time.h>
#ifdef _OPENMP
#include <omp.h>
#endif // _OPENMP

template <class T> void random_matrix(T *&mat, bool is_binary, const domain &d) {
  srand(time(NULL));
  T mod = is_binary ? 2 : std::numeric_limits<T>::max();
  for (size_t i = 0; i < d.vol; ++i)
    mat[i] = rand() % mod;
  if (is_binary) {
    for (size_t i = 0; i < d.vol; ++i)
      mat[i] *= std::numeric_limits<T>::max();
  }
}

// Record the execution time of some code, in milliseconds.
#define DECLARE_TIMING(s)                                                      \
  double timeStart_##s;                                                        \
  double timeDiff_##s;                                                         \
  double timeTally_##s = 0;                                                    \
  int countTally_##s = 0
#define START_TIMING(s) timeStart_##s = omp_get_wtime()
#define STOP_TIMING(s)                                                         \
  timeDiff_##s = omp_get_wtime() - timeStart_##s;                              \
  timeTally_##s += timeDiff_##s;                                               \
  countTally_##s++
#define GET_TIMING(s) timeDiff_##s
#define GET_AVERAGE_TIMING(s)   (double)(countTally_##s ? timeTally_##s/ (double)countTally_##s  : 0
#define CLEAR_AVERAGE_TIMING(s)                                                \
  timeTally_##s = 0;                                                           \
  countTally_##s = 0

#endif // __BENCH_HPP__
