/*
 * Copyright 2018, Theodore Chabardes
 * <theodore.chabardes@mines-paristech.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _BUTTON_HOLE_H_
#define _BUTTON_HOLE_H_

#include "concurrent_linked_list.hpp"
#include "concurrent_union_find.hpp"

template <class labelT> struct BHData {
  BHData(const BHData<labelT> &) = delete;
  BHData() = delete;
  BHData(size_t nbr) {
    has_overflown = (nbr + 1 > std::numeric_limits<labelT>::max());
    _size = (has_overflown) ? std::numeric_limits<labelT>::max() + 1 : nbr + 1;
    tail = (has_overflown) ? 0 : nbr + 1;

    BCL = new concurrent_linked_list<size_t>
        *[std::numeric_limits<labelT>::max() + 1];
    for (size_t i = 1; i < _size; ++i) {
      BCL[i] = new concurrent_linked_list<size_t>();
    }
  }

  ~BHData() {
        for (size_t i = 1; i < _size; ++i) {
          delete BCL[i];
        }
        delete[] BCL;
    
  }

  void print_self() {
    for (size_t i = 1; i < _size; ++i) {
      //      if (i==set.Find(i)) {
      std::cout << (int)i << " <=> " << (int)set.Find(i) << std::endl;
      BCL[i]->print_self();
      std::cout << std::endl;
      //      }
    }
    std::cout << std::endl;
  }

  size_t size() { return _size; }

  labelT push_back() {
    labelT atomic_value;
#ifdef _OPENMP
#pragma omp atomic capture
#endif // _OPENMP
    atomic_value = tail++;
    if (atomic_value == 0) {
      atomic_value++;
#ifdef _OPENMP
#pragma omp atomic write
#endif // _OPENMP
      has_overflown = true;
      _size = std::numeric_limits<labelT>::max() + 1;
    }

    if (!has_overflown) {
#ifdef _OPENMP
#pragma omp atomic write
#endif // _OPENMP
      _size = _size + 1;
      BCL[atomic_value] = new concurrent_linked_list<size_t>();
    }

    return atomic_value;
  }

  concurrent_linked_list<size_t> **BCL;
  concurrent_union_find<labelT> set;

private:
  bool has_overflown;
  size_t _size;
  labelT tail;
};

#endif // _BUTTON_HOLE_H_
