/*
 * Copyright 2018, Theodore Chabardes
 * <theodore.chabardes@mines-paristech.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "connexity.hpp"

// Convert argument number to structuring_element instance.
structuring_element get_structuring_element(const se_type &c) {
  structuring_element se;
  switch (c) {
  // Cross connexity in 2D
  case SE_CROSS:
    se.odd = false;
    se.addOffset(1, 0, 0);
    se.addOffset(0, 1, 0);
    se.addOffset(-1, 0, 0);
    se.addOffset(0, -1, 0);
    break;
  // Hexagonal connexity in 2D
  case SE_HEX:
    se.odd = true;
    se.addOffset(1, 0, 0);
    se.addOffset(0, 1, 0);
    se.addOffset(-1, 0, 0);
    se.addOffset(0, -1, 0);
    se.addOffset(1, 1, 0);
    se.addOffset(1, -1, 0);
    break;
  // Square connexity in 2D
  case SE_SQUARE:
    se.odd = false;
    se.addOffset(1, 0, 0);
    se.addOffset(0, 1, 0);
    se.addOffset(-1, 0, 0);
    se.addOffset(0, -1, 0);
    se.addOffset(1, 1, 0);
    se.addOffset(1, -1, 0);
    se.addOffset(-1, -1, 0);
    se.addOffset(-1, 1, 0);
    break;
  // Cross connexity in 3D
  case SE_CROSS_3D:
    se.odd = false;
    se.addOffset(1, 0, 0);
    se.addOffset(0, 1, 0);
    se.addOffset(0, 0, 1);
    se.addOffset(-1, 0, 0);
    se.addOffset(0, -1, 0);
    se.addOffset(0, 0, -1);
    break;
  // Cube connexity in 3D
  case SE_CUBE:
    se.odd = false;
    se.addOffset(1, 0, 0);
    se.addOffset(0, 1, 0);
    se.addOffset(0, 0, 1);
    se.addOffset(-1, 0, 0);
    se.addOffset(0, -1, 0);
    se.addOffset(0, 0, -1);

    se.addOffset(1, 1, 0);
    se.addOffset(1, -1, 0);
    se.addOffset(-1, -1, 0);
    se.addOffset(-1, 1, 0);

    se.addOffset(1, 1, 1);
    se.addOffset(1, -1, 1);
    se.addOffset(-1, -1, 1);
    se.addOffset(-1, 1, 1);

    se.addOffset(1, 1, -1);
    se.addOffset(1, -1, -1);
    se.addOffset(-1, -1, -1);
    se.addOffset(-1, 1, -1);

    se.addOffset(1, 0, 1);
    se.addOffset(-1, 0, 1);
    se.addOffset(0, 1, 1);
    se.addOffset(0, -1, 1);

    se.addOffset(1, 0, -1);
    se.addOffset(-1, 0, -1);
    se.addOffset(0, 1, -1);
    se.addOffset(0, -1, -1);
    break;
  default:
    break;
  }
  return se;
}
