/*
 * Copyright 2018, Theodore Chabardes
 * <theodore.chabardes@mines-paristech.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _COUNTS_H_
#define _COUNTS_H_

#include "connexity.hpp"
#include "neighbors.hpp"
#include <cstdlib>

// Counts the number of lower neighbors as defined in
// section 3.1 The number of lower neighbors is ranging from 0 to se.o.size()
// (the size of the structuring element)
template <class T1>
uint8_t count_prec(const size_t &p, const T1 *in, const unsigned short *homotopy,
                   const domain &d, const structuring_element &se) {
  unsigned int result = 0;
  ITERATE_NEIGHBORS_BEGIN(p, d, se) {
    if (lower_neighbor_abs(_p, p, in, homotopy))
      ++result;
  }
  ITERATE_NEIGHBORS_END
  return result;
}

// Counts the number of lower neighbors for all pixel as defined in section 3.1
template <class T1>
void create_counts_prec(const T1 *in, const unsigned short *homotopy, uint8_t *counts,
                        const domain &d, const structuring_element &se) {
#ifdef _OPENMP
#pragma omp for
#endif // _OPENMP
  for (size_t p = 0; p < d.vol; p++) {
    if (counts[p] == 0) {
      counts[p] = count_prec(p, in, homotopy, d, se);
    }
  }
}

#endif // _COUNTS_H_
