/*
 * Copyright 2018, Theodore Chabardes
 * <theodore.chabardes@mines-paristech.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _NEIGHBORS_H_
#define _NEIGHBORS_H_

#include <cassert>
#include <cstdlib>

// A macro to increase the readibility of the code.
// It performs boundaries check.
#define ITERATE_NEIGHBORS_BEGIN(A, DOMAIN, SE)                                 \
  {                                                                            \
    assert(DOMAIN.h > 0 && DOMAIN.w > 0);                                      \
    size_t x, y, z;                                                            \
    bool oddLine;                                                              \
                                                                               \
    x = A % DOMAIN.w;                                                          \
    y = (A % DOMAIN.line_len) / DOMAIN.w;                                      \
    z = A / DOMAIN.line_len;                                                   \
    oddLine = SE.odd && y % 2;                                                 \
                                                                               \
    for (unsigned char pts = 0; pts < SE.o.size(); ++pts) {                    \
      size_t _x = x + SE.o[pts].x;                                             \
      size_t _y = y + SE.o[pts].y;                                             \
      size_t _z = z + SE.o[pts].z;                                             \
      if (oddLine)                                                             \
        _x += (_y + 1) % 2;                                                    \
      if (_x < DOMAIN.w && _y < DOMAIN.h && _z < DOMAIN.d) {                   \
        size_t _p = _x + _y * DOMAIN.w + _z * DOMAIN.line_len;

#define ITERATE_NEIGHBORS_END                                                  \
  }                                                                            \
  }                                                                            \
  }

template <class T>
inline bool upper_neighbor(size_t _p, size_t p, const T *in, const unsigned short *homotopy) {
  return in[_p] > in[p] || (in[_p] == in[p] && homotopy[_p] > homotopy[p]);
}

template <class T>
inline bool level_neighbor(size_t _p, size_t p, const T *in, const unsigned short *homotopy) {
  return in[_p] == in[p] || (in[_p] == in[p] && homotopy[_p] == homotopy[p]);
}

template <class T>
inline bool lower_neighbor(size_t _p, size_t p, const T *in, const unsigned short *homotopy) {
  return in[_p] < in[p] || (in[_p] == in[p] && homotopy[_p] < homotopy[p]);
}

template <class T>
inline bool lower_or_level_neighbor(size_t _p, size_t p, const T *in, const unsigned short *homotopy) {
  return in[_p] < in[p] || (in[_p] == in[p] && homotopy[_p] <= homotopy[p]);
}

template <class T>
inline bool lower_neighbor_abs(size_t _p, size_t p, const T *in,
                              const unsigned short *homotopy) {
  return in[_p] < in[p] ||
         (in[_p] == in[p] && (homotopy[_p] < homotopy[p] ||
                              (homotopy[_p] == homotopy[p] && _p < p)));
}

template <class T>
inline bool upper_neighbor_abs(size_t _p, size_t p, const T *in,
                              const unsigned short *homotopy) {
  return in[_p] > in[p] ||
         (in[_p] == in[p] && (homotopy[_p] > homotopy[p] ||
                              (homotopy[_p] == homotopy[p] && _p > p)));
}

#endif // _NEIGHBORS_H_
