/*
 * Copyright 2017, Theodore Chabardes
 * <theodore.chabardes@mines-paristech.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "test_class.hpp"
#include "../src/paraws.hpp"

#define RED(txt) "\033[1;31m" << txt << "\033[0m"
#define GREEN(txt) "\033[1;32m" << txt << "\033[0m"

// Construct a new test with given input and structuring element.
test_class::test_class(unsigned char i[], unsigned char t[], const size_t &w,
                       const size_t &h, const se_type &c)
    : width(w), height(h) {
  in = new unsigned char[w * h];
  markers = NULL;
  truth = new unsigned char[w * h];
  out = new unsigned char[w * h];

  se = get_structuring_element(c);

  memcpy(in, i, w * h * sizeof(unsigned char));
  memcpy(truth, t, w * h * sizeof(unsigned char));
}

// Construct a new test with given input, markers and structuring element.
test_class::test_class(unsigned char i[], unsigned char m[], unsigned char t[],
                       const size_t &w, const size_t &h, const se_type &c)
    : width(w), height(h) {
  in = new unsigned char[w * h];
  markers = new unsigned char[w * h];
  truth = new unsigned char[w * h];
  out = new unsigned char[w * h];

  se = get_structuring_element(c);

  memcpy(in, i, w * h * sizeof(unsigned char));
  memcpy(markers, m, w * h * sizeof(unsigned char));
  memcpy(truth, t, w * h * sizeof(unsigned char));
}

// Desallocation.
test_class::~test_class() {
  delete[] in;
  delete[] truth;
  delete[] out;
}

// Perform the test and compare to the provided ground truth.
bool test_class::run() {
  if (markers == NULL)
    paraws(in, out, width, height, se, true);
  else
    paraws(in, markers, out, width, height, se, true);

  for (size_t i = 0; i < width * height; ++i)
    if (out[i] != truth[i])
      return false;
  return true;
}

// If the test did not pass, then the output is printed.
void test_class::print() {
  std::cout << "data size : [" << (int)width << "x" << (int)height << "]"
            << std::endl;

  for (size_t i = 0; i < height; ++i) {
    for (size_t j = 0; j < width; ++j)
      std::cout << (int)in[i * width + j] << "\t";
    std::cout << std::endl;
  }
  std::cout << std::endl;
  for (size_t i = 0; i < height; ++i) {
    for (size_t j = 0; j < width; ++j) {
      if (truth[i * width + j] != out[i * width + j])
        std::cout << RED((int)out[i * width + j]) << "/"
                  << GREEN((int)truth[i * width + j]) << "\t";
      else
        std::cout << (int)out[i * width + j] << "\t";
    }
    std::cout << std::endl;
  }
  std::cout << std::endl;
}
